--
-- $Id: PathUtils.hs,v 1.2 2005/12/03 14:59:23 aamine Exp $
--
-- Copyright (c) 2003-2005 Minero Aoki
--
-- This program is free software.
-- You can distribute/modify this program under the terms of
-- the GNU LGPL, Lesser General Public License version 2.1.
-- For details of the GNU LPGL, see the file "COPYING".
--

module PathUtils
    (joinPath, basename, dirname, components, ancestors)
where

import Data.List (intersperse)

joinPath :: [String] -> String
joinPath [] = "/"
joinPath [""] = "/"
joinPath cs = join "/" cs

basename :: String -> String
basename "/"  = "/"
basename path = last $ split '/' path

ancestors :: String -> [String]
ancestors path = case ancestors' path of
                   ("":xs) -> reverse $ "/" : xs
                   xs      -> reverse xs
  where
    ancestors' = scanl1 (\a b -> joinPath [a,b]) . components

components :: String -> [String]
components = split '/'

dirname "/" = "/"
dirname path = case (components path) of
                 []       -> ""
                 [_]      -> "."
                 cs@(_:_) -> (joinPath . reverse . tail . reverse) cs

split :: Eq a => a -> [a] -> [[a]]
split sep [] = []
split sep xs = x : split sep y'
               where (x,y) = break (==sep) xs
                     y' = case y of
                            [] -> []
                            _ -> tail y

join :: String -> [String] -> String
join sep = concat . intersperse sep
